#!/usr/bin/env python

from __future__ import print_function
import argparse
import os
import sys
import json
import normalize_az
import get_analysis
import add_feedback
import get_ita


def normalize(args):
  results = normalize_az.normalize_az(
      args.api_url,
      args.api_token,
      args.customer_access_key_id,
      args.customer_secret_access_key,
      args.region)

  if results is None:
    return 1
  else:
    print(json.dumps(results, indent=4))
    return 0


def analysis(args):
  results = get_analysis.get_analysis(
      args.api_url,
      args.api_token,
      args.normalized_az_list,
      args.instance_type_list,
      args.price_list,
      args.days)

  if results is None:
    return 1
  else:
    print(json.dumps(results, indent=4))
    return 0


def feedback(args):
  results = add_feedback.add_feedback(
      args.api_url,
      args.api_token,
      args.normalized_az,
      args.instance_type,
      args.bid_time,
      args.response_delay,
      args.num_instances_requested,
      args.bid_price,
      args.status,
      args.reason)

  if results is None:
    return 1
  else:
    print(json.dumps(results, indent=4))
    return 0


def instance_type_availability(args):
  results = get_ita.get_ita(
      args.api_url,
      args.api_token,
      args.regions)

  if results is None:
    return 1
  else:
    print(json.dumps(results, indent=4))
    return 0


def main():
  parser = argparse.ArgumentParser()
  parser.add_argument("--api-url",
      default=os.getenv("SPOT_API_URL", "http://localhost:5000"),
      help="""The API URL for the spot prediction service. You can either pass
      it through the command line or use the environment variable SPOT_API_URL.
      If none of these is used, the api url defaults to
      http://localhost:5000""")
  parser.add_argument("--api-token",
      default=os.getenv("SPOT_API_TOKEN"),
      help="""The API TOKEN for the spot prediction service. You can either
      pass it through the command line or use the environment variable
      SPOT_API_TOKEN.  This is required""")

  subparsers = parser.add_subparsers(title="sub commands", dest="subcommand")

  normalize_az_parser = subparsers.add_parser("normalize-az",
      description="""This command takes your aws credentials plus a region and
      returns the mapping from availability zones in your account to normalized
      availability zones used by the spot prediction service.""")
  normalize_az_parser.set_defaults(func=normalize)
  normalize_az_parser.add_argument("customer_access_key_id",
      help="""The aws_access_key_id for your account""")
  normalize_az_parser.add_argument("customer_secret_access_key",
      help="""The aws_secret_access_key for your account""")
  normalize_az_parser.add_argument("region",
      help="""The aws region for which you want to get the mapping""")

  get_analysis_parser = subparsers.add_parser("get-analysis",
      description="""This command takes a list of normalized
      availability zones, an ordered list of instance_type, a corresponding
      ordered list of bid_price and an optional number of days and returns the
      result of analysis by the spot prediction service""")
  get_analysis_parser.set_defaults(func=analysis)
  get_analysis_parser.add_argument("--normalized-az-list", required=True,
      nargs="+", help="list of normalized availability zones")
  get_analysis_parser.add_argument("--instance-type-list", required=True,
      nargs="+", help="ordered list of instance types")
  get_analysis_parser.add_argument("--price-list", required=True, nargs="+",
      help="ordered list of bid price corresponding to instance types")
  get_analysis_parser.add_argument("--days", type=int,
      help="the number of days of history to analyze")

  add_feedback_parser = subparsers.add_parser("add-feedback",
      description="""This method can be used to provide feedback to the spot
      predition service.""")
  add_feedback_parser.set_defaults(func=feedback)
  add_feedback_parser.add_argument("--normalized-az", required=True,
      help="the normalized availability zone where the spot request was made.")
  add_feedback_parser.add_argument("--instance-type", required=True,
      help="the instance type which was requested")
  add_feedback_parser.add_argument("--bid-time", required=True,
      help="the time when the spot request was made")
  add_feedback_parser.add_argument("--response-delay", required=True,
      help="""the number of seconds it took for aws to reply to the spot
      request (with success/failure)""")
  add_feedback_parser.add_argument("--num-instances-requested", required=True,
      help="the number of instances requested")
  add_feedback_parser.add_argument("--bid-price", required=True,
      help="the price you bid for")
  add_feedback_parser.add_argument("--status", choices=["success", "failure"],
      required=True, help="whether the request was a success or failure")
  add_feedback_parser.add_argument("--reason",
      help="if the request was a failure, the reason for the failure")

  get_ita_parser = subparsers.add_parser("get-instance-type-availability",
      description="""This method can be used to find out which instance types
      are available in which availability zones.""")
  get_ita_parser.set_defaults(func=instance_type_availability)
  get_ita_parser.add_argument("--regions", required=True,
      nargs="+", metavar="REGION",
      help="the regions in which availability is to be tested.")

  args = parser.parse_args()
  if args.api_token is None:
    print("No API Token provided. You can either pass it through the command-"
          "line (--api-token) or use the environment variable SPOT_API_TOKEN.",
          file=sys.stderr)
    sys.exit(2)
  sys.exit(args.func(args))


if __name__ == "__main__":
  main()
