#!/usr/bin/env python

from __future__ import print_function
import sys
import boto.ec2


def match_az(reference_access_key_id, reference_secret_access_key,
    customer_access_key_id, customer_secret_access_key, region):
  """
  This method takes two sets of aws_access_key_id and aws_secret_access_key
  and a region and returns a mapping from availability zones in one account to
  availability zones in reference account.
  """
  reference_ec2 = boto.ec2.connect_to_region(region,
      aws_access_key_id=reference_access_key_id,
      aws_secret_access_key=reference_secret_access_key)

  customer_ec2 = boto.ec2.connect_to_region(region,
      aws_access_key_id=customer_access_key_id,
      aws_secret_access_key=customer_secret_access_key)

  if reference_ec2 is None:
    print("Invalid region name provided.", file=sys.stderr)
    return None

  try:
    print("Fetching history from AWS...", file=sys.stderr)

    rsph = reference_ec2.get_spot_price_history(product_description="Linux/UNIX")
    csph = customer_ec2.get_spot_price_history(product_description="Linux/UNIX")
  except boto.exception.EC2ResponseError as e:
    print("Getting history from AWS failed.", file=sys.stderr)
    print("Status Code: %s" % e.status, file=sys.stderr)
    print("Reason: %s" % e.reason, file=sys.stderr)
    print("Body: %s" % e.body, file=sys.stderr)
    return None

  print("Fetching history complete.", file=sys.stderr)

  results = {}

  mapping = {}
  for rsp in rsph:
    if (rsp.timestamp, rsp.instance_type, rsp.price) in mapping:
      # We only use unique values for comparison.
      del mapping[rsp.timestamp, rsp.instance_type, rsp.price]
    else:
      mapping[rsp.timestamp, rsp.instance_type, rsp.price] = rsp.availability_zone

  for csp in csph:
    if (csp.timestamp, csp.instance_type, csp.price) in mapping:
      # We got a similar tuple using reference credentials, so we can match.
      ref_az = mapping[csp.timestamp, csp.instance_type, csp.price]
      if csp.availability_zone in results:
        if results[csp.availability_zone][0] == ref_az:
          confirmations = results[csp.availability_zone][1] + 1
          results[csp.availability_zone] = ref_az, confirmations
        else:
          # We have a negative confirmation! Can this happen?
          pass
      else:
        results[csp.availability_zone] = ref_az, 1

  for key in results.keys():
    if results[key][1] < 10:
      # Number of confirmations are too low, probably not a right mapping.
      # Remove it. These type of instances can occur when there is a AZ in
      # customer account which is not there in reference account.
      del results[key]

  return results


if __name__ == "__main__":

  reference_access_key_id = raw_input("Enter aws_access_key_id for reference account: ")
  reference_secret_access_key = raw_input("Enter aws_secret_access_key for reference account: ")

  customer_access_key_id = raw_input("Enter aws_access_key_id for customer account: ")
  customer_secret_access_key = raw_input("Enter aws_secret_access_key for customer account: ")

  region = raw_input("Enter region: ")

  results = match_az(reference_access_key_id, reference_secret_access_key,
      customer_access_key_id, customer_secret_access_key, region)

  import json
  print(json.dumps(results, indent=4))
